'''
towniesim.job
=============
'''
from random import gauss


class MetaJob(type):
    JOBS = {}

    def __new__(cls, name, bases, dct):
        if name == 'Job':
            return super(MetaJob, cls).__new__(cls, name, bases, dct)
        for key in ('CAN_OWN', 'SUPERVISOR', 'MILITARY', 'WATER'):
            dct[key] = dct.get(key, False)
        for key in ('LEVEL',):
            dct[key] = dct.get(key, 1)
        new_cls = super(MetaJob, cls).__new__(cls, name, bases, dct)
        MetaJob.JOBS[dct['NAME']] = new_cls
        return new_cls


class Job(metaclass=MetaJob):
    COUNT = 0
    OBJS = {}

    @classmethod
    def next(cls):
        cls.COUNT += 1
        return cls.COUNT

    def __init__(self, wealth=None, location=None, pay=None):
        pay = pay or self.WAGE
        self.wealth = wealth or self.LEVEL
        self.pay = int(max(gauss(pay, pay / 10), 2))
        self.location = location
        self.id = Job.next()
        self.idstr = '{}#{}'.format(self.NAME, self.id).lower()
        Job.OBJS[self.idstr] = self

    @property
    def location(self):
        from .building import Building
        if self._location is None:
            return None
        return Building.OBJS[self._location]

    @location.setter
    def location(self, val):
        from .building import Building
        if isinstance(val, Building):
            self._location = val.idstr
        else:
            self._location = val

    @classmethod
    def get_jobs(cls):
        return MetaJob.JOBS.values()

    @classmethod
    def get_job(cls, name):
        return MetaJob.JOBS[name]

    def __repr__(self):
        return '{name} [{pay}c/day]'.format(
            name=self.NAME.title(), wealth=self.wealth, pay=self.pay
        )


class ApothecaryJob(Job):
    NAME = 'apothecary'
    WAGE = 40
    CAN_OWN = True
    LEVEL = 1


class ButcherJob(Job):
    NAME = 'butcher'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1


class HerbalistJob(Job):
    NAME = 'herbalist'
    WAGE = 40
    CAN_OWN = True
    LEVEL = 1


class ClothierJob(Job):
    NAME = 'clothier'
    WAGE = 30
    CAN_OWN = True
    LEVEL = 1


class ShoemakerJob(Job):
    NAME = 'shoemaker'
    WAGE = 30
    CAN_OWN = True
    LEVEL = 1


class SpinsterJob(Job):
    NAME = 'spinster'
    WAGE = 5
    LEVEL = 1


class BakerJob(Job):
    NAME = 'baker'
    WAGE = 20
    CAN_OWN = True
    LEVEL = 1


class BarberJob(Job):
    NAME = 'barber'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 2


class BlacksmithJob(Job):
    NAME = 'blacksmith'
    WAGE = 100
    CAN_OWN = True
    LEVEL = 1


class ArmorerJob(Job):
    NAME = 'armorer'
    WAGE = 100
    CAN_OWN = True
    LEVEL = 1


class BowyerJob(Job):
    NAME = 'bowyer'
    WAGE = 50
    CAN_OWN = True
    LEVEL = 1


class AtilliatorJob(Job):
    NAME = 'atilliator'
    WAGE = 80
    CAN_OWN = True
    LEVEL = 1


class FletcherJob(Job):
    NAME = 'fletcher'
    WAGE = 20
    LEVEL = 1


class SailorJob(Job):
    NAME = 'sailor'
    WAGE = 5
    WATER = True
    LEVEL = 1


class MessengerJob(Job):
    NAME = 'messenger'
    WAGE = 2
    LEVEL = 2


class MinerJob(Job):
    NAME = 'miner'
    WAGE = 10
    LEVEL = 1
    INDUSTRIAL = True


class LaborerJob(Job):
    NAME = 'laborer'
    WAGE = 5
    LEVEL = 1


class CookJob(Job):
    NAME = 'cook'
    WAGE = 40
    CAN_OWN = True
    LEVEL = 1


class ServerJob(Job):
    NAME = 'server'
    WAGE = 3
    LEVEL = 1


class ArtistJob(Job):
    NAME = 'artist'
    WAGE = 50
    CAN_OWN = True
    LEVEL = 3


class CandlemakerJob(Job):
    NAME = 'candlemaker'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1


class CarpenterJob(Job):
    NAME = 'carpenter'
    WAGE = 60
    CAN_OWN = True
    LEVEL = 1


class MoneylenderJob(Job):
    NAME = 'moneylender'
    WAGE = 100
    CAN_OWN = True
    LEVEL = 3


class ClerkJob(Job):
    NAME = 'clerk'
    WAGE = 50
    LEVEL = 3


class PotterJob(Job):
    NAME = 'potter'
    WAGE = 20
    CAN_OWN = True
    LEVEL = 1


class LibrarianJob(Job):
    NAME = 'librarian'
    WAGE = 50
    CAN_OWN = True
    LEVEL = 2


class ScribeJob(Job):
    NAME = 'scribe'
    WAGE = 50
    CAN_OWN = True
    LEVEL = 2


class JewelerJob(Job):
    NAME = 'jeweler'
    WAGE = 100
    CAN_OWN = True
    LEVEL = 3


class MasonJob(Job):
    NAME = 'mason'
    WAGE = 20
    CAN_OWN = True
    LEVEL = 1


class GuardJob(Job):
    NAME = 'guard'
    WAGE = 20
    MILITARY = True
    LEVEL = 2


class GuardCaptainJob(Job):
    NAME = 'guardcaptain'
    WAGE = 100
    SUPERVISOR = True
    MILITARY = True
    LEVEL = 3


class ShipCaptainJob(Job):
    NAME = 'shipcaptain'
    WAGE = 100
    SUPERVISOR = True
    WATER = True
    LEVEL = 2


class FishermanJob(Job):
    NAME = 'fisherman'
    WAGE = 10
    WATER = True
    LEVEL = 1
    INDUSTRIAL = True


class FarmerJob(Job):
    NAME = 'farmer'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1
    INDUSTRIAL = True


class BartenderJob(Job):
    NAME = 'bartender'
    WAGE = 20
    CAN_OWN = True
    LEVEL = 1


class ForesterJob(Job):
    NAME = 'forester'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1
    INDUSTRIAL = True


class QuarrymanJob(Job):
    NAME = 'quarryman'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1
    INDUSTRIAL = True


class TailorJob(Job):
    NAME = 'tailor'
    WAGE = 30
    CAN_OWN = True
    LEVEL = 1


class WeaverJob(Job):
    NAME = 'weaver'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1


class HatmakerJob(Job):
    NAME = 'hatmaker'
    WAGE = 30
    CAN_OWN = True
    LEVEL = 2


class LocksmithJob(Job):
    NAME = 'locksmith'
    WAGE = 30
    CAN_OWN = True
    LEVEL = 2


class RooferJob(Job):
    NAME = 'roofer'
    WAGE = 20
    CAN_OWN = True
    LEVEL = 2


class TannerJob(Job):
    NAME = 'tanner'
    WAGE = 10
    CAN_OWN = True
    LEVEL = 1


class RugmakerJob(Job):
    NAME = 'rugmaker'
    WAGE = 50
    CAN_OWN = True
    LEVEL = 3



'''
towniesim.shop
==============
'''
from random import choice, uniform
from .job import *


class MetaBuilding(type):
    SHOPS = {}

    def __new__(cls, name, bases, dct):
        if name in ('Building', 'Residence'):
            return super(MetaBuilding, cls).__new__(cls, name, bases, dct)
        if 'JOBS' not in dct:
            dct['JOBS'] = {}
        if 'LEVEL' not in dct:
            dct['LEVEL'] = 0
        for key in ('MILITARY', 'WATER', 'INDUSTRIAL'):
            dct[key] = dct.get(key, False)
        new_cls = super(MetaBuilding, cls).__new__(cls, name, bases, dct)
        MetaBuilding.SHOPS[dct['NAME']] = new_cls
        return new_cls


class Workforce:

    def __init__(self):
        self.data = {}

    def __getitem__(self, val):
        from .npc import NPC
        if val.NAME in self.data:
            return [NPC.OBJS[x] for x in self.data[val.NAME]]
        else:
            return []

    def add(self, job_type, npc):
        self.data[job_type.NAME] = self.data.get(job_type.NAME, [])
        self.data[job_type.NAME].append(npc.idstr)

    def items(self):
        for name, ids in self.data.items():
            job = Job.get_job(name)
            yield (job, self[job])


class Building(metaclass=MetaBuilding):
    COUNT = 0
    OBJS = {}

    @classmethod
    def next(cls):
        cls.COUNT += 1
        return cls.COUNT

    @property
    def owner(self):
        from .npc import NPC
        if self._owner is None:
            return None
        return NPC.OBJS[self._owner]

    @owner.setter
    def owner(self, val):
        from .npc import NPC
        if isinstance(val, NPC):
            self._owner = val.idstr
        else:
            self._owner = val

    def __init__(self, owner=None, wealth=0, inventory=None, district=None,
                 **kwargs):
        self.owner = owner
        self.jobs = []
        self.wealth = wealth
        self.workers = Workforce()
        self.inventory = inventory or set()
        self.district = district
        self.id = Building.next()
        self.idstr = '{}#{}'.format(self.NAME, self.id).lower()
        Building.OBJS[self.idstr] = self
        self.name = self.NAME

    def count_missing_jobs(self):
        total = 0
        for job, mnmx in self.JOBS.items():
            num = len(self.workers[job])
            if num < mnmx[0]:
                total += mnmx[0] - num
        return total

    def new_job(self):
        required = []
        desired = []
        always_hiring = []
        for job, mnmx in self.JOBS.items():
            num = len(self.workers[job])
            if num < mnmx[0]:
                required.append((job, (mnmx[0] - num) / mnmx[0]))
            if mnmx[1] is None:
                always_hiring.append((job, num))
            # if none this crashes, so leave above
            elif num < mnmx[1]:
                desired.append((job, (mnmx[1] - num) / mnmx[1]))
        required = sorted(required, key=lambda x: 1, reverse=True)
        desired = sorted(desired, key=lambda x: 1, reverse=True)
        # least jobs goes to front
        always_hiring = sorted(always_hiring, key=lambda x: 1)
        if required:
            return required[0][0]
        if desired:
            # a third of the time they go to mine/quarry/forestry/pier/etc?
            if always_hiring and uniform(0, 1) < (1 / 3):
                return always_hiring[0][0]
            return desired[0][0]
        return None

    def hire_worker(self, npc):
        job_type = self.new_job()
        if job_type is None:
            return None
        if not npc.can_work():
            return None
        self.workers.add(job_type, npc)
        job = job_type(wealth=self.wealth, location=self)
        npc.job = job
        self.jobs.append(job)
        return job

    @classmethod
    def rand(cls, **kwargs):
        return choice(cls.filter(**kwargs))

    @classmethod
    def check_conditions(cls, bld, wealth=None, water=None, military=None,
                         industrial=None):
        if isinstance(bld, Building):
            bld = bld.__class__
        if wealth:
            if bld.LEVEL > wealth:
                return False
        if water is not None:
            if bld.WATER != water:
                return False
        if military is not None:
            if bld.MILITARY != military:
                return False
        if industrial is not None:
            if bld.INDUSTRIAL != industrial:
                return False
        return True

    @classmethod
    def filter(cls, **kwargs):
        return [
            x for x in cls.get_buildings()
            if cls.check_conditions(x, **kwargs)
        ]

    @classmethod
    def get_buildings(cls):
        return MetaBuilding.SHOPS.values()

    @classmethod
    def get_building(cls, name):
        return MetaBuilding.SHOPS[name]

    def __repr__(self):
        title = '{name}[{owner}]'.format(
            name=self.NAME,
            owner=self.owner.name,
        )
        s = []
        for job, workers in self.workers.items():
            s.append('#{}={}'.format(job.NAME, len(workers)))
        s = ', '.join(s)
        return '{}({})'.format(title, s)

    def dump(self):
        workers = ''
        for job, works in self.workers.items():
            for work in works:
                workers += '{jobid} - {worker!r} {workerid}\n'.format(
                    jobid=work.job.idstr,
                    worker=work,
                    workerid=work.idstr,
                )
        return '''{name}
ID: {idstr}
Owner: {owner!r}
Owner ID: {ownerid}
Workers:\n{workers}
'''.format(
            name=self.NAME,
            idstr=self.idstr,
            owner=self.owner,
            ownerid=self.owner.idstr,
            workers=workers,
        )


class BakeryBuilding(Building):
    NAME = 'bakery'
    LEVEL = 1
    COST = 2000
    JOBS = {BakerJob: (1, 4)}


class BlacksmithBuilding(Building):
    NAME = 'blacksmith'
    LEVEL = 1
    COST = 3000
    JOBS = {BlacksmithJob: (1, 4), ArmorerJob: (0, 2)}


class BowyerBuilding(Building):
    NAME = 'bowyer'
    LEVEL = 1
    JOBS = {BowyerJob: (1, 4), FletcherJob: (1, 4)}
    COST = 3000


class ArmorerBuilding(Building):
    NAME = 'armorer'
    LEVEL = 2
    JOBS = {ArmorerJob: (1, 4), BlacksmithJob: (0, 4)}
    COST = 6000


class AtilliatorJob(Building):
    NAME = 'atilliator'
    LEVEL = 2
    JOBS = {AtilliatorJob: (1, 4), BowyerJob: (0, 4), FletcherJob: (1, 4)}
    COST = 6000


class ButcherBuilding(Building):
    NAME = 'butchery'
    LEVEL = 1
    JOBS = {ButcherJob: (1, 4)}
    COST = 2000


class TailorBuilding(Building):
    NAME = 'tailor'
    LEVEL = 1
    JOBS = {
        TailorJob: (1, 4),
    }
    COST = 3000


class ClothierBuilding(Building):
    NAME = 'clothier'
    LEVEL = 1
    JOBS = {
        ClothierJob: (1, 4),
        ShoemakerJob: (0, 2),
        SpinsterJob: (0, 4),
        WeaverJob: (0, 2),
        TailorJob: (0, 2),
    }
    COST = 3000


class PotteryBuilding(Building):
    NAME = 'potter'
    LEVEL = 1
    JOBS = {PotterJob: (1, 4)}
    COST = 2000


class FishBuilding(Building):
    NAME = 'fishmonger'
    LEVEL = 1
    WATER = True
    JOBS = {FishermanJob: (1, 4)}
    COST = 2000


class ProduceBuilding(Building):
    NAME = 'produce'
    LEVEL = 1
    JOBS = {FarmerJob: (1, 4)}
    COST = 1500


class CarpentryBuilding(Building):
    NAME = 'carpenter'
    LEVEL = 1
    JOBS = {CarpenterJob: (1, 4)}
    COST = 2500


class ApothecaryBuilding(Building):
    NAME = 'apothecary'
    LEVEL = 2
    JOBS = {
        ApothecaryJob: (1, 4),
        HerbalistJob: (1, 4),
    }
    COST = 5000


class CandleBuilding(Building):
    NAME = 'candlemaker'
    LEVEL = 2
    JOBS = {CandlemakerJob: (1, 4)}
    COST = 5000


class LibraryBuilding(Building):
    NAME = 'library'
    LEVEL = 2
    JOBS = {LibrarianJob: (1, 12), ScribeJob: (0, 8)}
    COST = 8000


class TavernBuilding(Building):
    NAME = 'tavern'
    LEVEL = 2
    JOBS = {BartenderJob: (1, 4), CookJob: (1, 3), ServerJob: (1, 3)}
    COST = 7000


class InnBuilding(Building):
    NAME = 'inn'
    LEVEL = 2
    JOBS = {BartenderJob: (1, 8), CookJob: (1, 4), ServerJob: (1, 6)}
    COST = 10000


class PierTavernBuilding(Building):
    NAME = 'waterside-tavern'
    LEVEL = 1
    JOBS = {BartenderJob: (1, 4), CookJob: (1, 3), ServerJob: (1, 3)}
    WATER = True
    COST = 7500


class ArtBuilding(Building):
    NAME = 'gallery'
    LEVEL = 3
    JOBS = {ArtistJob: (1, 4)}
    COST = 10000


class BankBuilding(Building):
    NAME = 'bank'
    LEVEL = 3
    JOBS = {MoneylenderJob: (1, 4), ClerkJob: (1, 8)}
    COST = 15000


class BarberBuilding(Building):
    NAME = 'barbershop'
    LEVEL = 2
    JOBS = {BarberJob: (1, 4)}
    COST = 5000


class QuarryBuilding(Building):
    NAME = 'quarry'
    LEVEL = 1
    JOBS = {QuarrymanJob: (8, None)}
    INDUSTRIAL = True
    COST = 25000


class SawmillBuilding(Building):
    NAME = 'sawmill'
    LEVEL = 1
    JOBS = {ForesterJob: (8, None)}
    INDUSTRIAL = True
    COST = 25000


class MineBuilding(Building):
    NAME = 'mine'
    LEVEL = 1
    JOBS = {MinerJob: (8, None)}
    INDUSTRIAL = True
    COST = 25000


class PierBuilding(Building):
    NAME = 'pier'
    LEVEL = 1
    WATER = True
    JOBS = {
        FishermanJob: (8, None),
        SailorJob: (4, None),
        ShipCaptainJob: (1, None),
    }
    INDUSTRIAL = True
    COST = 25000


class JewelryBuilding(Building):
    NAME = 'jeweler'
    LEVEL = 2
    JOBS = {JewelerJob: (1, 4)}
    COST = 8000


class MasonryBuilding(Building):
    NAME = 'masonry'
    LEVEL = 1
    JOBS = {MasonJob: (1, 4)}
    COST = 1500


class HatmakerBuilding(Building):
    NAME = 'hatmaker'
    LEVEL = 2
    JOBS = {HatmakerJob: (1, 4)}
    COST = 3000


class LocksmithBuilding(Building):
    NAME = 'locksmith'
    LEVEL = 2
    JOBS = {LocksmithJob: (1, 4)}
    COST = 4000


class RooferBuilding(Building):
    NAME = 'roofer'
    LEVEL = 1
    JOBS = {RooferJob: (1, 4)}
    COST = 2000


class TanneryBuilding(Building):
    NAME = 'tanner'
    LEVEL = 1
    JOBS = {TannerJob: (1, 4)}
    COST = 2000


class RugmakerBuilding(Building):
    NAME = 'rugmaker'
    LEVEL = 3
    JOBS = {RugmakerJob: (1, 6)}
    COST = 7000



'''
namerator
=========

Name generator, by generating characters based on frequencies of letter patterns

Usage:
    $ python namerator.py -n 5 elf_names.txt
    egil-gonamilin
    taedhorilie
    lalorahinduil
    cirduiel
    amrothrie

'''
import sys
import os
from math import sqrt
from random import gauss, seed, choice
from collections import Counter

NAME_ROOT = os.path.join(os.path.dirname(__file__), 'names')


def load(path):
    with open(path) as f:
        return [x.strip() for x in f.readlines() if x.strip()]


def load_names():
    names = {}
    files = os.listdir(NAME_ROOT)
    for f in files:
        name, _ = os.path.splitext(f)
        path = os.path.join(NAME_ROOT, f)
        names[name.replace('_', '-')] = load(path)
    return names


def load_frequencies():
    freq = {}
    for fname, names in NAMES.items():
        freq[fname] = calc_frequencies(names)
    return freq


def chop_name(name):
    lst = [x for x in name.lower().strip().split()[0]]
    return ['', '', '', ''] + lst + ['$']


def inc_frequencies(chop, freqs):
    freqs[0] = freqs.get(0, Counter())
    for i in range(1, 5):
        freqs[i] = freqs.get(i, {})
    for i in range(3, len(chop)):
        a, b, c, d = chop[i-4], chop[i-3], chop[i-2], chop[i-1]
        char = chop[i]
        freqs[0].update([char])
        freqs[1][d] = freqs[1].get(d, Counter())
        freqs[1][d].update([char])
        freqs[2][c + d] = freqs[2].get(c + d, Counter())
        freqs[2][c + d].update([char])
        freqs[3][b + c + d] = freqs[3].get(b + c + d, Counter())
        freqs[3][b + c + d].update([char])
        freqs[4][a + b + c + d] = freqs[4].get(a + b + c + d, Counter())
        freqs[4][a + b + c + d].update([char])
    return freqs


def calc_frequencies(names):
    freqs = {'len': Counter()}
    for name in names:
        freqs['len'].update([len(name)])
        chop = chop_name(name)
        inc_frequencies(chop, freqs)
    return freqs


def combined_frequencies(freqs, last1, last2, last3, last4):
    freq = Counter()
    freq.update(freqs[1][last1])
    for i in range(5):
        freq.update(freqs[2].get(last2, []))
    for i in range(10):
        freq.update(freqs[3].get(last3, []))
    for i in range(25):
        freq.update(freqs[4].get(last4, []))
    return freq.most_common()


def choose(common, over=0, letter_freqs=None):
    lst = []
    for char, num in common:
        if over < 0 and char == '$':
            continue
        lst += [char] * num
    if over >= 0:
        lst += ['$'] * dict(common).get('$', 1) * (over + 1)
    if not lst:
        return choose(letter_freqs, over=over)
    return choice(lst)


def calc_gauss(lens):
    expanded = []
    for l, num in lens:
        expanded += [l] * num
    mean = sum(expanded) / len(expanded)
    sigma = sum((mean - x)**2 for x in expanded)
    sigma /= len(expanded) - 1
    sigma = sqrt(sigma)
    return mean, sigma


def generate(freqs):
    if isinstance(freqs, str):
        freqs = FREQS[freqs]
    name = ''
    last1 = ''
    last2 = ''
    last3 = ''
    last4 = ''
    shortest = min(x for x, y in freqs['len'].most_common())
    # longest = max(x for x, y in freqs['len'].most_common())
    mean, sigma = calc_gauss(freqs['len'].most_common())
    namelen = int(max(gauss(mean, sigma), shortest))
    while True:
        combined = combined_frequencies(freqs, last1, last2, last3, last4)
        next_letter = choose(combined, over=len(name) - namelen,
                             letter_freqs=freqs[0].most_common())
        if next_letter == '$':
            break
        name += next_letter
        last4 = last3 + next_letter
        last3 = last2 + next_letter
        last2 = last1 + next_letter
        last1 = next_letter
    return name


def make_name(race, gender=None):
    if not isinstance(race, str):
        race = race.NAME
    if gender is not None and race in ('human', 'halfling'):
        if race == 'human':
            race = 'human-' + gender
        elif race == 'halfling':
            race = 'halfling-' + gender
    if race == 'human-male':
        first = generate('human-first-male')
        last = generate('human-last')
        return '{} {}'.format(first, last).title()
    elif race == 'human-female':
        first = generate('human-first-female')
        last = generate('human-last')
        return '{} {}'.format(first, last).title()
    elif race == 'human-first-male':
        return generate('human-first-male').title()
    elif race == 'human-first-female':
        return generate('human-first-female').title()
    elif race == 'human2':
        return generate('human-last').title()
    elif race == 'elf':
        return '{} {}'.format(generate('elf'), generate('elf')).title()
    elif race == 'half-elf':
        first = generate('elf')
        last = generate('human-last')
        return '{} {}'.format(first, last).title()
    elif race == 'dwarf':
        first = generate('dwarf')
        last = choice(NAMES['dwarf-part1']) + choice(NAMES['dwarf-part2'])
        return '{} {}'.format(first, last).title()
    elif race == 'gnome':
        first = generate('dwarf')
        last = choice(NAMES['dwarf-part1']) + generate('dwarf-part2')
        return '{} {}'.format(first, last).title()
    elif race == 'dwarf-first':
        return generate('dwarf').title()
    elif race == 'dwarf-last':
        return choice(NAMES['dwarf-part1']) + choice(NAMES['dwarf-part2'])
    elif race == 'orc':
        return generate('orc').title()
    elif race == 'half-orc':
        first = generate('orc').title()
        last = generate('human-last')
        return '{} {}'.format(first, last).title()
    elif race == 'halfling-male':
        first = generate('human-first-male')
        last = generate('halfling-last')
        return '{} {}'.format(first, last).title()
    elif race == 'halfling-female':
        first = generate('human-first-female')
        last = generate('halfling-last')
        return '{} {}'.format(first, last).title()
    else:
        raise NotImplementedError('cant generate name for {}'.format(race))


def main():
    import argparse
    import json
    parser = argparse.ArgumentParser()
    parser.add_argument('path')
    parser.add_argument('--num', '-n', type=int, default=10)
    parser.add_argument('--output', '-o')
    parser.add_argument('--freq-output', '-f')
    args = parser.parse_args()
    if os.path.isfile(args.path):
        names = load(args.path)
        freqs = calc_frequencies(names)
    else:
        sys.exit('Please pass a path to the name file or one of "human", '
                 '"elf", "dwarf" or "orc"')
    if args.freq_output:
        dump = {}
        dump['len'] = freqs['len'].most_common()
        dump['frequency'] = freqs[0].most_common()
        dump['first'] = {k: v.most_common() for k, v in freqs[1].items()}
        dump['second'] = {k: v.most_common() for k, v in freqs[2].items()}
        dump['third'] = {k: v.most_common() for k, v in freqs[3].items()}
        dump['fourth'] = {k: v.most_common() for k, v in freqs[4].items()}
        with open(args.freq_output, 'w') as f:
            json.dump(dump, f, indent=4)
        print('Dumped frequencies to {args.freq_output}'.format(args=args))
    names = set(names)
    ct = 0
    if args.output is None:
        while ct < args.num:
            name = generate(freqs)
            if name in names:
                seed()
                continue
            names.add(name)
            print(name)
            ct += 1
        return
    with open(args.output, 'w') as f:
        while ct < args.num:
            name = generate(freqs)
            if name in names:
                seed()
                continue
            names.add(name)
            f.write(name + '\n')
            ct += 1
            if ct % 100 == 0:
                perc = float(ct) / args.num
                print('{:.2%} done'.format(perc))


NAMES = load_names()
FREQS = load_frequencies()


if __name__ == '__main__':
    main()



'''
Generate a town based on a configuration

pseudocode:

    For each district, tally up the commercial and industrial and residential
    For each commercial, create a random building by wealth
    For each industrial, create a random building by wealth
    For each residential, create an NPC family
      For each new family, assign an empty workplace of appropriate wealth
          Find first job which CAN_OWN
              assign to older members and set them as owner
          random choice for other members of family
    For each unowned commercial, create new family and say they live there

Later:
    military
    government
'''
import os
import re
import yaml
from random import uniform, shuffle
from itertools import chain
from collections import defaultdict
from .building import Building
from .residence import Residence
from .family import Family
from .util import random_from_list


RE_FILENAME = re.compile(r'[^a-zA-Z0-9 -_]')


def make_filename(name):
    name = name.lower().replace(' ', '_').replace('-', '_')
    return RE_FILENAME.sub('', name)


class TownConfig:

    def __init__(self, path):
        self.path = path
        with open(path) as f:
            self.config = yaml.load(f)
        self.validate_config()

    def __repr__(self):
        return 'TownConfig({!r})'.format(self.config)

    @property
    def districts(self):
        return self.config['districts']

    @property
    def races(self):
        return self.config.get('race')

    @property
    def industry(self):
        return self.config.get('industry')

    def validate_config(self):
        if 'districts' not in self.config:
            raise ValueError('town config must specify at least one district')
        if not self.races:
            self.config['race'] = {
                'human': 60,
                'elf': 12,
                'dwarf': 12,
                'half-elf': 6,
                'half-orc': 4,
                'gnome': 3,
                'halfling': 3,
            }
        if not self.industry:
            self.config['industry'] = {'farmer': 10}


class District:

    def __init__(self, name=None, config=None, default_races=None):
        self._name = name
        self.config = config
        self.default_races = default_races
        self.buildings = []
        self.residences = []
        self.families = []
        self.npcs = []

    def __repr__(self):
        return (
            'District({!r}, #buildings={}, #residences={}, #families={})'
            .format(self._name, len(self.buildings), len(self.residences),
                    len(self.families))
        )

    def create(self):
        wealth = self.config.get('wealth') or 3
        water = self.config.get('water') or False
        military = self.config.get('military') or False
        races = self.config.get('race', self.default_races)
        comm = self.config.get('commercial', 0)
        rezs = self.config.get('residences', 0)
        ind = self.config.get('industrial', {})
        for i in range(comm):
            bld = Building.rand(
                wealth=wealth,
                water=water,
                military=military,
                industrial=False,
            )(wealth=wealth, district=self._name)
            self.buildings.append(bld)
        for type_name, ct in ind.items():
            typ = Building.get_building(type_name)
            for i in range(ct):
                bld = typ(wealth=wealth)
                self.buildings.append(bld)
        for i in range(rezs):
            typ = Residence.rand(wealth=self.config.get('wealth') or 2)
            bld = typ(wealth=wealth)
            race = random_from_list(races)
            fam = Family(race=race, wealth=wealth)
            bld.owner = fam.oldest()
            bld.owner.owned_residences += [bld]
            self.residences.append(bld)
            self.families.append(fam)
            self.npcs.extend(list(fam.members()))

    def auction_buildings(self):
        fams = sorted(self.families, key=lambda x: x.gold, reverse=True)
        unowned = sorted(
            (x for x in self.buildings if x.owner is None),
            key=lambda x: x.COST,
            reverse=True,
        )
        for i in range(2):
            for un in unowned:
                for fam in fams:
                    if fam.can_afford(un):
                        fam.buy(un)
                        break
                fams = sorted(self.families, key=lambda x: x.gold, reverse=True)
            unowned = sorted(
                (x for x in self.buildings if x.owner is None),
                key=lambda x: x.COST,
                reverse=True,
            )

    def assign_jobs(self):
        npcs = [x for x in self.npcs if x.can_work() and x.job is None]
        for npc in npcs:
            owns = [
                x for x in npc.owned_buildings
                if x.count_missing_jobs() > 0
            ]
            for bld in owns:
                job = bld.hire_worker(npc)
                if job is not None:
                    break
            if npc.job is not None:
                continue
            buildings = sorted(
                self.buildings,
                key=lambda x: x.count_missing_jobs(),
                reverse=True,
            )
            buildings = [x for x in buildings if x.count_missing_jobs() > 0]
            if not buildings:
                break
            buildings[0].hire_worker(npc)

    def dump(self, out):
        fname = make_filename(self._name) + '.txt'
        path = os.path.join(out, fname)
        with open(path, 'w') as f:
            f.write('name: {}\n\n'.format(self._name))
            f.write('#occupants: {}\n'.format(len(self.npcs)))
            f.write('#families: {}\n'.format(len(self.families)))
            f.write('#residences: {}\n'.format(len(self.residences)))
            f.write('#businesses: {}\n\n'.format(len(self.buildings)))
            f.write('Residences\n----------\n\n')
            for i, res in enumerate(self.residences):
                f.write('Residence {}\n'.format(i + 1))
                f.write(res.dump() + '\n\n')
            f.write('NPCs\n----\n\n')
            for fam in self.families:
                fstr = '{} "{}"'.format(fam.idstr, fam.name)
                f.write(fstr + '\n' + len(fstr) * '-' + '\n\n')
                for npc in fam.members():
                    f.write(npc.dump() + '\n\n')
            f.write('\nBUILDINGs\n---------\n\n')
            for bld in self.buildings:
                f.write(bld.dump() + '\n\n')


class Town:

    def __init__(self, config):
        if isinstance(config, TownConfig):
            self.config = config
        else:
            self.config = TownConfig(config)
        self.districts = []
        self.create()
        self.districts = sorted(
            self.districts,
            key=lambda x: (x.config.get('wealth'), x._name),
        )

    def __repr__(self):
        s = ''
        for i, d in enumerate(self.districts):
            s += '  {}: {!r}\n'.format(i, d)
        return 'Town(\n{})'.format(s)

    def __getitem__(self, i):
        return self.districts[i]

    @property
    def families(self):
        return list(chain(*[x.families for x in self.districts]))

    @property
    def npcs(self):
        return list(chain(*[x.npcs for x in self.districts]))

    @property
    def buildings(self):
        return list(chain(*[x.buildings for x in self.districts]))

    @property
    def residences(self):
        return list(chain(*[x.residences for x in self.districts]))

    def finish_assigning_jobs(self):
        npcs = [x for x in self.npcs if x.can_work() and x.job is None]
        for npc in npcs:
            buildings = sorted(
                self.buildings,
                key=lambda x: x.count_missing_jobs(),
                reverse=True,
            )
            buildings = [x for x in buildings if x.count_missing_jobs() > 0]
            if not buildings:
                break
            buildings[0].hire_worker(npc)
        buildings = [x for x in buildings if x.count_missing_jobs() > 0]
        for bld in buildings:
            raise ValueError('{!r} failed to get enough workers!'.format(bld))
        npcs = [x for x in self.npcs if x.can_work() and x.job is None]
        buildings = self.buildings
        for npc in npcs:
            shuffle(buildings)
            for building in self.buildings:
                job = building.hire_worker(npc)
                if job is not None:
                    break
            else:
                raise ValueError('{!r} failed to get a job!'.format(npc))

    def auction_buildings(self):
        fams = sorted(self.families, key=lambda x: x.gold, reverse=True)
        unowned = sorted(
            (x for x in self.buildings if x.owner is None),
            key=lambda x: x.COST,
            reverse=True,
        )
        while unowned:
            bought = False
            for un in unowned:
                for fam in fams:
                    if fam.can_afford(un):
                        fam.buy(un)
                        bought = True
                        break
                fams = sorted(self.families, key=lambda x: x.gold, reverse=True)
            unowned = sorted(
                (x for x in self.buildings if x.owner is None),
                key=lambda x: x.COST,
                reverse=True,
            )
            if not bought:
                for fam in fams:
                    if uniform(0, 1) < 0.1:
                        fam.gold *= 1.5

    def create(self):
        for name, dist in self.config.districts.items():
            new = District(name=name, config=dist,
                           default_races=self.config.races)
            self.districts.append(new)
            new.create()
        for dist in self.districts:
            dist.auction_buildings()
        worked = defaultdict(list)
        for bld in self.buildings:
            worked[0].append(bld)
        worked[0].sort(key=lambda x: x.wealth, reverse=True)
        self.auction_buildings()
        for dist in self.districts:
            dist.assign_jobs()
        self.finish_assigning_jobs()

    def output_residents(self):
        for npc in self.npcs:
            npc.output()

    def dump(self, output_dir):
        from yaml.representer import Representer
        from .race import MetaRace
        from .building import MetaBuilding
        from .job import MetaJob
        from .residence import MetaResidence
        for meta in (MetaRace, MetaBuilding, MetaJob, MetaResidence):
            Representer.add_representer(meta, Representer.represent_name)
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        raw_path = os.path.join(output_dir, 'raw.yml')
        with open(raw_path, 'w') as f:
            s = yaml.dump({'town': self}, default_flow_style=False)
            f.write(s)
        print('Saved raw town data to {}'.format(raw_path))
        for district in self.districts:
            district.dump(output_dir)

    def reload(self):
        from .building import Building
        from .residence import Residence
        from .family import Family
        from .job import Job
        from .npc import NPC
        Building.COUNT = len(self.buildings)
        Residence.COUNT = len(self.residences)
        Family.COUNT = len(self.families)
        NPC.COUNT = len(self.npcs)
        Job.COUNT = len([x for x in self.npcs if x._job is not None])
        for bld in self.buildings:
            Building.OBJS[bld.idstr] = bld
            for job in bld.jobs:
                Job.OBJS[job.idstr] = job
        for res in self.residences:
            Residence.OBJS[res.idstr] = res
        for fam in self.families:
            Family.OBJS[fam.idstr] = fam
        for npc in self.npcs:
            NPC.OBJS[npc.idstr] = npc



'''
towniesim.race
==============
'''
from enum import Enum
from random import choice
from .util import gauss_mm, le_switch


class Age(Enum):
    child = 'child'
    adult = 'adult'
    middle = 'middle-aged'
    old = 'old'
    venerable = 'venerable'

    @classmethod
    def random_age(cls, parents=None):
        parents = [x.get_age() for x in parents if x]
        choices = [x for x in cls]
        for parent in parents:
            for age in choices[:]:
                if age >= parent:
                    choices.remove(age)
        return choice(choices)

    @classmethod
    def random_parent(cls):
        return choice([Age.adult, Age.middle, Age.old])

    @classmethod
    def random_notchild(cls):
        return choice([Age.adult, Age.middle, Age.old, Age.venerable])

    @classmethod
    def random_child(cls):
        return choice([Age.child, Age.adult])

    @classmethod
    def random(cls):
        return choice([x for x in cls])

    def _compare(self, other):
        numeric = {
            self.__class__.child.value: 0,
            self.__class__.adult.value: 1,
            self.__class__.middle.value: 2,
            self.__class__.old.value: 3,
            self.__class__.venerable.value: 4,
        }
        if numeric[self.value] == numeric[other.value]:
            return 0
        if numeric[self.value] < numeric[other.value]:
            return -1
        if numeric[self.value] > numeric[other.value]:
            return 1

    def __lt__(self, other):
        return self._compare(other) < 0

    def __gt__(self, other):
        return self._compare(other) > 0

    def __eq__(self, other):
        return self._compare(other) == 0

    def __le__(self, other):
        return self._compare(other) <= 0

    def __ge__(self, other):
        return self._compare(other) >= 0

    def __ne__(self, other):
        return self._compare(other) != 0


class MetaRace(type):
    RACES = {}

    def __new__(cls, name, bases, dct):
        if name == 'Race':
            return super(MetaRace, cls).__new__(cls, name, bases, dct)
        new_cls = super(MetaRace, cls).__new__(cls, name, bases, dct)
        MetaRace.RACES[dct['NAME']] = new_cls
        return new_cls


class Race(metaclass=MetaRace):

    @classmethod
    def get_races(cls):
        return MetaRace.RACES.values()

    @classmethod
    def get_race(cls, race):
        return MetaRace.RACES[race]

    @classmethod
    def get_age(cls, age_num):
        age = le_switch(cls.AGE, age_num)
        if age is None:
            return Age.venerable
        return age

    @classmethod
    def random_age(cls, age=None):
        ages = sorted(cls.AGE.items())
        if age is None:
            age = choice(a for a in Age)
        if age is Age.child:
            return gauss_mm(ages[0][0] / 2, ages[0][0] / 3, 0, ages[0][0])
        elif age is Age.adult:
            mid = (ages[0][0] + ages[1][0]) / 2
            return gauss_mm(mid, mid / 2, ages[0][0] + 1, ages[1][0])
        elif age is Age.middle:
            mid = (ages[1][0] + ages[2][0]) / 2
            return gauss_mm(mid, mid / 2, ages[1][0] + 1, ages[2][0])
        elif age is Age.old:
            mid = (ages[2][0] + ages[3][0]) / 2
            return gauss_mm(mid, mid / 2, ages[2][0] + 1, ages[3][0])
        elif age is Age.venerable:
            mid = (ages[3][0] + 1) + (ages[0][0] / 2)
            return gauss_mm(mid, mid / 2, ages[3][0] + 1, None)


class Human(Race):
    NAME = 'human'
    AGE = {15: Age.child, 34: Age.adult, 52: Age.middle, 69: Age.old}
    RACISM = 0


class HalfOrc(Race):
    NAME = 'half-orc'
    AGE = {12: Age.child, 29: Age.adult, 44: Age.middle, 59: Age.old}
    RACISM = 10


class Elf(Race):
    NAME = 'elf'
    AGE = {40: Age.child, 174: Age.adult, 262: Age.middle, 349: Age.old}
    RACISM = 0


class Dwarf(Race):
    NAME = 'dwarf'
    AGE = {30: Age.child, 124: Age.adult, 187: Age.middle, 249: Age.old}
    RACISM = 0


class HalfElf(Race):
    NAME = 'half-elf'
    AGE = {20: Age.child, 61: Age.adult, 92: Age.middle, 124: Age.old}
    RACISM = 5


class Gnome(Race):
    NAME = 'gnome'
    AGE = {25: Age.child, 99: Age.adult, 262: Age.middle, 349: Age.old}
    RACISM = 0


class Halfling(Race):
    NAME = 'halfling'
    AGE = {18: Age.child, 49: Age.adult, 74: Age.middle, 99: Age.old}
    RACISM = -5



'''
clouddistrict

Generate occupants and occupations for your custom village, town or city! (for D&D purposes or related) (please do not imbibe clouddistrict while under the influence of other medications)
'''

__title__ = 'clouddistrict'
__version__ = '0.0.4'
__all__ = ()
__author__ = 'Johan Nestaas <johannestaas@gmail.com>'
__license__ = 'GPLv3+'
__copyright__ = 'Copyright 2017 Johan Nestaas'

from sys import exit
import yaml
from .town import Town


def main():
    import argparse
    parser = argparse.ArgumentParser()
    subs = parser.add_subparsers(dest='cmd')

    p = subs.add_parser('create')
    p.add_argument('output_dir')
    p.add_argument('--config', '-c', default='town.yml')

    p = subs.add_parser('load')
    p.add_argument('raw_town_data')

    args = parser.parse_args()

    if args.cmd == 'create':
        try:
            town = Town(args.config)
        except ValueError as e:
            print(str(e))
            print('You might need to put less businesses and more residents.')
            exit(1)
        town.dump(args.output_dir)
        print('Dumped all files to directory: {}'.format(args.output_dir))
    elif args.cmd == 'load':
        with open(args.raw_town_data) as f:
            data = yaml.load(f.read())
        town = data['town']
        town.reload()
        print('loaded town (nothing implemented yet, will have shell later)')
    else:
        parser.print_usage()
        exit(1)


if __name__ == '__main__':
    main()



'''
towniesim.npc
=============

Generate a random NPC

from .job import (ApothecaryJob, ButcherJob, HerbalistJob, ClothierJob,
                  ShoemakerJob, SpinsterJob, BakerJob, BarberJob,
                  BlacksmithJob, ArmorerJob, BowyerJob, AtilliatorJob,
                  FletcherJob, CookJob, ServerJob, ArtistJob, CandlemakerJob,
                  CarpenterJob, MoneylenderJob, ClerkJob, PotterJob,
                  LibrarianJob, ScribeJob, FishermanJob, FarmerJob,
                  BartenderJob, ForesterJob, Job)
'''
from enum import Enum
from random import choice, gauss
from .namerator import make_name
from .race import Race, Age
from .util import (
    random_from_list, friendly_wealth, friendly_racism, friendly_age,
)

CLASSES = ['fighter', 'wizard', 'rogue', 'cleric', 'ranger', 'barbarian',
           'druid', 'monk', 'bard', 'paladin', 'sorcerer', 'warlock']


class Memory:

    def __init__(self):
        self.reaction = 0


class NPC:
    COUNT = 0
    OBJS = {}

    @classmethod
    def next(cls):
        cls.COUNT += 1
        return cls.COUNT

    def __init__(self, race=None, name=None, level=1, klass=None, job=None,
                 age=None, gender=None, lover=None, family=None, wealth=1):
        self.id = NPC.next()
        self.idstr = 'NPC#{}'.format(self.id).lower()
        NPC.OBJS[self.idstr] = self
        self.chars, self.alignment = Character.rand()
        race = race or choice(['human', 'half-orc', 'elf', 'dwarf', 'half-elf',
                               'gnome', 'halfling'])
        self.gender = gender or choice(['male', 'female'])
        self.race = race
        self.name = name or make_name(race=self.race, gender=self.gender)
        self.level = level
        self.wealth = wealth
        self.klass = klass
        self.job = job
        self.age = self.race.random_age(age=age)
        self.agestr = self.race.get_age(self.age).value
        self.lover = lover
        self.family = family
        self.owned_residences = []
        self.owned_buildings = []
        self.bond = choice(BONDS)
        self.ideal = self.choose_ideal()
        self.trait = choice(TRAITS)
        self.flaw = choice(FLAWS)
        self.init_racism()

    def choose_ideal(self):
        choice_keys = ['any'] + self.alignment
        choices = []
        for key in IDEALS:
            if key in choice_keys:
                choices.extend(IDEALS[key])
        return choice(choices)

    @property
    def race(self):
        return Race.get_race(self._race)

    @race.setter
    def race(self, val):
        if hasattr(val, 'NAME'):
            self._race = val.NAME
        else:
            self._race = val

    def __repr__(self):
        age = self.race.get_age(self.age).value
        mid = ' '
        if self.job:
            mid = ' {!r} '.format(self.job)
        elif self.klass:
            mid = ' {!r} '.format(self.klass)
        return '{age} {gender} {race}{mid}"{name}" wealth={wealth}'.format(
            age=age,
            race=self.race.NAME.title(),
            gender=self.gender,
            mid=mid,
            name=self.name,
            wealth=self.family.wealth,
        )

    @property
    def lover(self):
        if self._lover is None:
            return None
        return NPC.OBJS[self._lover]

    @lover.setter
    def lover(self, val):
        if isinstance(val, NPC):
            self._lover = val.idstr
        else:
            self._lover = val

    @property
    def family(self):
        from .family import Family
        if self._family is None:
            return None
        return Family.OBJS[self._family]

    @family.setter
    def family(self, val):
        from .family import Family
        if isinstance(val, Family):
            self._family = val.idstr
        else:
            self._family = val

    @property
    def job(self):
        from .job import Job
        if self._job is None:
            return None
        return Job.OBJS[self._job]

    @job.setter
    def job(self, val):
        from .job import Job
        if isinstance(val, Job):
            self._job = val.idstr
        else:
            self._job = val

    @property
    def owned_buildings(self):
        from .building import Building
        blds = []
        for bld in self._owned_buildings:
            blds.append(Building.OBJS[bld])
        return blds

    @owned_buildings.setter
    def owned_buildings(self, val):
        self._owned_buildings = []
        for bld in val:
            self._owned_buildings.append(bld.idstr)

    @property
    def owned_residences(self):
        from .residence import Residence
        blds = []
        for bld in self._owned_residences:
            blds.append(Residence.OBJS[bld])
        return blds

    @owned_residences.setter
    def owned_residences(self, val):
        self._owned_residences = []
        for bld in val:
            self._owned_residences.append(bld.idstr)

    def output(self):
        s = '{!r}'.format(self)
        if self.job:
            s += '\nworks at {!r}'.format(self.job)
        s += '\n{}\n'.format('\n'.join(self.chars))
        print(s)

    def init_racism(self):
        self.racism = {}
        for r in Race.get_races():
            self.racism[r.NAME] = gauss(0, 7)
            self.racism[r.NAME] += r.RACISM
        self.racism[self.race.NAME] -= 15
        self.racism[self.race.NAME] = min(self.racism[self.race.NAME], 15)

    def take_last_name(self, other):
        last = other.get_last_name()
        if last is None:
            raise ValueError('No name for significant other: {!r}'.format(other))
            return None
        myspl = self.name.split()
        self.name = '{} {}'.format(myspl[0], last).title()
        return last

    def short_family(self):
        children = None
        if self.family.children:
            children = ', '.join([
                x.name.split()[0] for x in self.family.children
            ])
        mother = self.family.mother and self.family.mother.name.split()[0]
        father = self.family.father and self.family.father.name.split()[0]
        if self.family.mother is self:
            if children:
                if father:
                    return 'married to {}, mother of {}'.format(
                        father, children,
                    )
                return 'mother of {}'.format(children)
            if father:
                return 'married to {}'.format(father)
            return 'no partner or children'
        if self.family.father is self:
            if children:
                if mother:
                    return 'married to {}, father of {}'.format(
                        mother, children,
                    )
                return 'father of {}'.format(children)
            if mother:
                return 'married to {}'.format(mother)
            return 'no partner or children'
        if self.family.children and self in self.family.children:
            children = self.family.children
            children.remove(self)
            if not children:
                if father:
                    if mother:
                        return 'son of {} and {}, no siblings'.format(
                            father, mother)
                    return 'father is {}, no siblings'.format(father)
                if mother:
                    return 'mother is {}, no siblings'.format(mother)
                return 'no partner or siblings'
            else:
                children = ', '.join([
                    x.name.split()[0] for x in children
                ])
                if father:
                    if mother:
                        return 'son of {} and {}, siblings are {}'.format(
                            father, mother, children,
                        )
                    return 'father is {}, siblings are {}'.format(
                        father, children)
                if mother:
                    return 'mother is {}, siblings are {}'.format(
                        mother, children)
                return 'siblings are {}'.format(children)

    def long_r(self):
        return (
            'NPC(name="{s.name}", race="{s.race!r}", age={s.age}, '
            'level={s.level}, klass={s.klass!r}, job={s.job!r})'
            .format(s=self)
        )

    def dump(self):
        desc = ', '.join(self.chars)
        racism = []
        for race, val in self.racism.items():
            f = friendly_racism(val)
            if f is None:
                continue
            racism.append('{} {}s'.format(f, race))
        racism = ', '.join(racism)
        if not racism:
            racism = 'neutral'
        s = '''{name}
id: {id}
Level {level} {gender} {race}
Alignment: {alignment[0]} {alignment[1]}
Age: {age} ({agestr})
Job ID: {jobid}
Job: {job} {jobdistrict}
Class: {klass!r}
Wealth class: {wealth}
Description: {desc}
Racial reaction: {racism}
owns residences: {owns_residences}
owns businesses: {owns_buildings}
Family ID: {familyid}
Family: {family}
Trait: {trait}
Bond: {bond}
Ideal: {ideal}
Flaw: {flaw}
'''.format(
            id=self.idstr,
            name=self.name,
            level=self.level,
            alignment=['true', 'neutral'] if self.alignment == ['neutral', 'neutral'] else self.alignment,
            gender=self.gender,
            race=self.race.NAME,
            age=friendly_age(self.age),
            agestr=self.get_age().value,
            job=self.job or '',
            jobdistrict=(
                self.job and 'in the {}'.format(self.job.location.district)
            ) or '',
            jobid=(self.job or '') and self.job.idstr,
            klass=self.klass,
            wealth=friendly_wealth(self.family.wealth),
            desc=desc,
            racism=racism,
            owns_residences=', '.join([b.idstr for b in self.owned_residences]),
            owns_buildings=', '.join([b.idstr for b in self.owned_buildings]),
            familyid=self.family.idstr,
            family=self.short_family(),
            trait=self.trait,
            bond=self.bond,
            ideal=self.ideal,
            flaw=self.flaw,
        )
        return s

    def get_last_name(self):
        spl = self.name.split()
        if len(spl) == 1:
            raise ValueError('No name for self: {!r}'.format(self))
            return None
        return spl[-1].title()

    def get_age(self):
        return self.race.get_age(self.age)

    def can_work(self):
        return self.get_age() not in (Age.child, Age.venerable)


class QualityStrength(Enum):
    average = None
    very = 'very'
    extremely = 'extremely'

    @classmethod
    def prepend(cls, s):
        c = random_from_list({
            None: 85,
            'very': 12,
            'extremely': 3,
        })
        if c:
            return '{} {}'.format(c, s)
        else:
            return s


BODY_QUALITIES = {
    'strength': ('weak', 'strong'),
    'intelligence': ('stupid', 'smart'),
    'agility': ('clumsy', 'graceful'),
    'weight': ('thin', 'fat'),
    'looks': ('ugly', 'pretty'),
}

PERSONALITIES = {
    'religion': ('atheistic', 'superstitious', 'spiritual', 'religious'),
    'sympathy': ('empathetic', 'self-centered'),
    'vanity': ('narcississtic', 'modest'),
    'naivete': ('gullible', 'cunning'),
    'courage': ('cowardly', 'courageous'),
    'laziness': ('lazy', 'ambitious'),
    'stubborness': ('stubborn', 'compliant'),
    'calmness': ('anxious', 'calm'),
    'introvert': ('introverted', 'extroverted'),
    'rudeness': ('rude', 'polite'),
    'generous': ('stingy', 'generous'),
    'playfulness': ('serious', 'playful'),
    'romantic': ('cold', 'romantic'),
    'chaotic': ('chaotic', 'lawful'),
    'evil': ('evil', 'good'),
    'skeptic': ('skeptical', 'trusting'),
    'private': ('private', 'public'),
    'gloomy': ('gloomy', 'joyful'),
    'security': ('insecure', 'confident'),
    'paranoia': ('paranoid', 'stable'),
    'submission': ('submissive', 'dominating'),
    'appreciate': ('unappreciative', 'gracious'),
    'volume': ('soft-spoken', 'obnoxious'),
    'optimist': ('pessimistic', 'optimistic'),
}


class Character:

    @classmethod
    def rand(cls):
        chars = []
        alignment = ['neutral', 'neutral']
        body_ct = random_from_list({
            0: 40,
            1: 50,
            2: 10,
        })
        mood_ct = random_from_list({
            1: 60,
            2: 35,
            3: 5,
        })
        if body_ct + mood_ct < 2:
            mood_ct += 1
        can_use_body = set(BODY_QUALITIES.keys())
        can_use_pers = set(PERSONALITIES.keys())
        for i in range(body_ct):
            key = choice(list(can_use_body))
            can_use_body.remove(key)
            b = choice(BODY_QUALITIES[key])
            s = QualityStrength.prepend(b)
            chars.append(s)

        for i in range(mood_ct):
            key = choice(list(can_use_pers))
            can_use_pers.remove(key)
            b = choice(PERSONALITIES[key])
            if b == 'evil':
                alignment[1] = 'evil'
            elif b == 'good':
                alignment[1] = 'good'
            elif b == 'chaotic':
                alignment[0] = 'chaotic'
            elif b == 'lawful':
                alignment[0] = 'lawful'
            s = QualityStrength.prepend(b)
            chars.append(s)
        return chars, alignment


BONDS = [
    'I would lay down my life for the people I served with.',
    'By preserving the natural order I ensure the continuation of the good in the world',
    'The goal of a life of study is the betterment of oneself.',
    'Solitude and contemplation are paths toward mystical or magical power.',
    'We have to take care of each other, because no one else is going to do it.',
    "My house's alliance with another noble family must be sustained at all costs.",
    'Peace between individuals is the most pleasant state of affairs.',
    'Performance of one’s duty is the highest honor',
    'I seek to preserve a sacred text that my enemies consider heretical and seek to destroy.',
    "Someone stole my precious instrument, and someday I'll get it back.",
    "There's no good pretending to be something I'm not.",
    "It is each person's responsibility to make the most happiness for the whole tribe.",
    'In life as in war, the stronger force wins.',
    'My stature in regard to myself and others is paramount.',
    'The old order is nothing compared to what will follow.',
    "I've been searching my whole life for the answer to a certain question.",
    'I sponsor an orphanage to keep others from enduring what I was forced to endure.',
    'My city, nation, or people are all that matter.',
    'I protect those who cannot protect themselves.',
    'Emotions must not cloud our logical thinking.',
    'I suffer awful visions of a coming disaster and will do anything to prevent it.',
    'Action against all odds and reality represents the strength of the individual in relation to the world.',
    'If I dishonor myself, I dishonor my whole clan.',
    "I come from a noble family, and one day I'll reclaim my lands and title from those who stole them from me.",
    'I am defined purely by how capable I am.',
    "I seek to prove myself worthy of my god's favor by matching my actions against his or her teachings.",
    'My honor is my life.',
    'I will someday get revenge on the corrupt temple hierarchy who branded me a heretic.',
    'The world is at constant war with itself, and by arbitrating between those parts outside and inside myself, I am more in tune with it',
    'There is no greater shame than a betrayal.',
    'If I do not correctly understand the world, then I cannot realistically engage with it.',
    'If I engage in the world through close examination of my place and my actions within it, I am able to be most fruitful',
    "My town or city is my home, and I'll fight to defend it.",
    'By boldly behaving in a way contrary to expectations, I cause the world to be more exciting.',
    'The rich need to be shown what life and death are like in the gutters.',
    'I would die to recover an ancient artifact of my faith that was lost long ago.',
    'In order for life to remain valuable, there must be obstacles to overcome and problems to face',
    'I have an ancient text that holds terrible secrets that must not fall into the wrong hands.',
    'Nothing and no one can steer me away from my higher calling.',
    'My gifts are meant to be shared with all, not used for my own benefit.',
    'The strongest are meant to rule. ',
    "I hope to one day rise to the top of my faith's religious hierarchy. ",
    'I work hard to be the best there is at my craft.',
    'The most rewarding part of humanoid experience lies exploration and excitement',
    'The thing that keeps a ship together is mutual respect between captain and crew.',
    'The ship is most important--crewmates and captains come and go.',
    'To feel emotions - love, hate, greed, compassion - is the most humanoid thing that the individual can do.',
    "I'm only in it for the money.",
    "I never target people who can't afford to lose a few coins. ",
    'No one should get preferential treatment before the law, and no one is above the law. ',
    "I'm determined to make something of myself.",
    'I will become the greatest thief that ever lived.',
    "I'm going to prove that I'm worthy of a better life.",
    "There's a spark of good in everyone.",
    'Knowledge is the path to power and domination',
    'Nothing is more important that achieving one’s goals.',
    'It is the duty of all civilized people to strengthen the bonds of community and the security of civilization. ',
    'Art should reflect the soul; it should come from within and reveal who we really are.',
    'My loyalty to my sovereign is unwavering.',
    'In order to achieve my ends, I must be able to control the environment around me.',
    'I was cheated of my fair share of the profits, and I want to get my due.',
    'Material goods come and go. Bonds of friendship last forever.',
    'People need to be united so that they do not destroy each other.',
    'We must help bring about the changes the gods are constantly working in the world. ',
    "My life's work is a series of tomes related to a specific field of lore.",
    'I fight for those who cannot fight for themselves.',
    'Someone saved my life on the battlefield. To this day, I will never leave a friend behind.',
    'I do what I must and obey just authority.',
    'When people follow orders blindly they embrace a kind of tyranny.',
    'Emotions must not cloud our sense of what is right and true, or our logical thinking.',
    'The act of living is also the act of changing, and by not changing one denies his very living.',
    'I have a family, but I have no idea where they are. One day, I hope to see them again.',
    'An injury to the unspoiled wilderness of my home is an injury to me.',
    'My talents were given to me so that I could use them to benefit the world.',
    'I worked the land, I love the land, and I will protect the land.',
    'I sold my soul for knowledge. I hope to do great deeds and win it back.',
    'Doing acts that make the world a better place is the best way for me to spend my life.',
    'I owe me life to the priest who took me in when my parents died.',
    'When I perform, I make the world better than it was. ',
    "Someday I'll own my own ship and chart my own destiny. ",
    'I always try to help those in need, no matter what the personal cost.',
    "A powerful person killed someone I love. Some day soon, I'll have my revenge.",
    'In a harbor town, I have a paramour whose eyes nearly stole me from the sea.',
    'Life is like the seasons, in constant change, and we must change with it. ',
    'I owe my guild a great debt for forging me into the person I am today.',
    'The sea is freedom--the freedom to go anywhere and do anything.',
    "I owe everything to my mentor--a horrible person who's probably rotting in jail somewhere.",
    "Somewhere out there I have a child who doesn't know me. I'm making the world better for him or her.",
    'My family, clan, or tribe is the most important thing in my life, even when they are far from me.',
    'I will do anything to protect the temple where I served.',
    'I am a free spirit--no one tells me what to do.',
    "I'm trying to pay off an old debt I owe to a generous benefactor.",
    'Everyone should be free to pursue his or her livelihood.',
    'I will face any challenge to win the approval of my family.',
    'The obscene and the unusual are beautiful.',
    "I help people who help me--that's what keeps us alive.",
    'I entered seclusion to hide from the ones who might still be hunting me. I must someday confront them.',
    'If I can attain more power, no one will tell me what to do. ',
    'People need to be united so that they can all achieve common goals.',
    "I idolize a hero of the old tales and measure my deeds against that person's.",
    "No one else is going to have to endure the hardships I've been through.",
    "I like seeing the smiles on people's faces when I perform. That's all that matters. ",
    'I wish my childhood sweetheart had come with me to pursue my destiny.',
    'True action requires the individual to be sure of their beliefs.',
    'I will do whatever it takes to become wealthy. ',
    'If you know yourself, there’s nothing left to know.',
    'I will bring terrible wrath down on the evildoers who destroyed my homeland.',
    'I would do anything for the other members of my old troupe.',
    'I search for inspiring sights and extraordinary circumstances',
    'The stories, legends, and songs of the past must never be forgotten.',
    'What is beautiful points us beyond itself toward what is true.',
    'The greater community should produce things: art, commerce, experience, etc...',
    "I'm committed to my crewmates, not to ideals. ",
    'I am in love with the heir of a family that my family despises.',
    "I'll always remember my first ship.",
    "I'm only in it for the money and fame. ",
    'People deserve to be treated with dignity and respect.',
    'I am the last of my tribe, and it is up to me to ensure their names enter legend.',
    'I owe my survival to another urchin who taught me to live on the streets.',
    'Should my discovery come to light, it could bring ruin to the world.',
    'Those who fight beside me are those worth dying for.',
    "I'm loyal to my captain first, everything else second.",
    "I'm guilty of a terrible crime. I hope I can redeem myself for it.",
    "I'll never forget the crushing defeat my company suffered or the enemies who dealt it.",
    'If I become strong, I can take what I want--what I deserve.',
    'Respect is due to me because of my position, but all people regardless of station deserve to be treated with dignity.',
    'One day I will return to my guild and prove that I am the greatest artisan of them all.',
    'It is my duty to provide children to sustain my tribe.',
    'I entered seclusion because I loved someone I could not have.',
    'Chains are meant to be broken, as are those who would forge them. Tyrants must not be allowed to oppress the people. ',
    'A proud noble once gave me a horrible beating, and I will take my revenge on any bully I encounter.',
    'One true measure of my accomplishments is the recognition of others',
    'When an act brings another pain or destruction, it must be answered in appropriate proportion.',
    'All people, rich or poor, deserve respect. ',
    'Everything I do is for the common people.',
    'My personal ideals and beliefs are small compared to what I can pdo by devoting myself to a cause or individual',
    'Our lot is to lay down our lives in defense of others.',
    'Life is more enjoyable when it is organized in clearly marked boxes.',
    'The world is in need of new ideas and bold action.',
    'It is my duty to protect my students.',
    'I fleeced the wrong person and must work to ensure that this individual never crosses paths with me or those I care about.',
    'My isolation gave me great insight into a great evil that only I can destroy.',
    'I will do anything to prove myself superior to me hated rival.',
    'I must prove that I can handle myself without the coddling of my family. ',
    "I'm still seeking the enlightenment I pursued in my seclusion, and it still eludes me.",
    'It is my duty to protect and care for the people beneath me. ',
    "I escaped my life of poverty by robbing an important person, and I'm wanted for it.",
    'We all do the work, so we all share in the rewards.',
    'Nothing is more important that the other members of my family.',
    "I'm a predator, and the other ships on the sea are my prey.",
    'Nothing should fetter the infinite possibility inherent in all existence.',
    'I steal from the wealthy so that I can help people in need.',
    'Freedom of action and thought are the primary concerns of any civilized society.',
    'In order to make my life worth living, I must explore who I am and learn about the world',
    "I'm committed to the people I care about, not to ideals. ",
    'Chaotic forces in the universe serve only to disrupt real progress and development.',
    'My tools are symbols of my past life, and I carry them so that I will never forget my roots.',
    'It is my duty to respect the authority of those above me, just as those below me must respect mine.',
    'Live and Let Live. ',
    'I must be able to control myself first if I wish to honestly affect the world.',
    'The workshop where I learned my trade is the most important place in the world to me.',
    'Something important was taken from me, and I aim to steal it back.',
    'I work to preserve a library, university, scriptorium, or monastery.',
    'I want to be famous, whatever it takes.',
    'The natural world is more important than all the constructs of civilization.',
    'Someone I loved died because of a mistake I made. That will never happen again.',
    'I must earn glory in battle, for myself and my clan.',
    'I will get revenge on the evil forces that destroyed my place of business and ruined my livelihood.',
    'The alignment of one’s goals to an established system allows me to do greater things than I could on my own.',
    'By discovering more about Vthe world around me, I am able to more accurately align my views with the reality of the universe.',
    'I owe a debt I can never repay to the person who took pity on me.',
    'There is something pure and honest at the moment when something new is learned or knowledge of the universe is gained',
    'The outcome of my life is less important than how beautiful it was.',
    'My ill-gotten gains go to support my family.',
    'Flights of fancy and listening to a good story are far more important than the physical world.',
    'Nothing is more important than the other members of my hermitage, order, or association.',
    "I don't steal from others in the trade.",
    'Meddling in the affairs of others only causes trouble.',
    'The new and the untimely are beautiful in and of themselves, because they are original creations in the world.',
    'My instrument is my most treasured possession, and it reminds me of someone I love.',
    'The common folk must see me as a hero of the people.',
    'The path to power and self-improvement is through knowledge.',
    'Inquiry and curiosity are the pillars of progress.',
    'I distribute money I acquire to the people who really need it. ',
    'The ancient traditions of worship and sacrifice must be preserved and upheld. ',
    'I never run the same con twice. ',
    "I created a great work for someone, and then found them unworthy to receive it. I'm still looking for someone worthy.",
    "I swindled and ruined a person who didn't deserve it. I seek to atone for my misdeeds but might never be able to forgive myself.",
    'The low are lifted up, and the high and mighty are brought down. Change is the nature of things.',
    "I'm loyal to my friends, not to any ideals, and everyone else can take a trip down the Styx for all I care.  ",
    "I pursue wealth to secure someone's love.",
    'Ruthless pirates murdered my captain and crewmates, plundered our ship, and left me to die. Vengeance will be mine.',
    'My life has meaning because I am able to freely enjoy myself in the world.',
    'Blood runs thicker than water.',
]

IDEALS = {
    'any': [
        "Aspiration. I seek to prove my self worthy of my god's favor by matching my actions against his or her teachings.",
        "Aspiration. I'm determined to make something of myself.",
        'Honesty. Art should reflect the soul; it should come from within and reveal who we really are.',
        'Destiny. Nothing and no one can steer me away from my higher calling.',
        'Aspiration. I work hard to be the best there is at my craft.',
        "Self-Knowledge. If you know yourself, there're nothing left to know.",
        'Family. Blood runs thicker than water.',
        'Glory. I must earn glory in battle, for myself and my clan.',
        "Aspiration. Someday I'll own my own ship and chart my own destiny.",
        'Nation. My city, nation, or people are all that matter.',
        "Aspiration. I'm going to prove that I'm worthy of a better life.",
    ],
    'chaotic': [
        'Change. We must help bring about the changes the gods are constantly working in the world.',
        'Independence. I am a free spirit--no one tells me what to do.',
        'Creativity. I never run the same con twice.',
        'Freedom. Chains are meant to be broken, as are those who would forge them.',
        'Creativity. The world is in need of new ideas and bold action.',
        'Freedom. Tyrants must not be allowed to oppress the people.',
        'Freedom. Everyone should be free to pursue his or her livelihood.',
        'Free Thinking. Inquiry and curiosity are the pillars of progress.',
        'Independence. I must prove that I can handle myself without the coddling of my family.',
        'Change. Life is like the seasons, in constant change, and we must change with it.',
        'No Limits. Nothing should fetter the infinite possibility inherent in all existence.',
        'Freedom. The sea is freedom--the freedom to go anywhere and do anything.',
        'Independence. When people follow orders blindly they embrace a kind of tyranny.',
        'Change. The low are lifted up, and the high and mighty are brought down. Change is the nature of things.',
    ],
    'evil': [
        'Greed. I will do whatever it takes to become wealthy.',
        "Greed. I'm only in it for the money and fame.",
        'Might. If I become strong, I can take what I want--what I deserve.',
        "Greed. I'm only in it for the money.",
        'Power. Solitude and contemplation are paths toward mystical or magical power.',
        'Power. If I can attain more power, no one will tell me what to do.',
        'Might. The strongest are meant to rule.',
        'Power. Knowledge is the path to power and domination.',
        "Master. I'm a predator, and the other ships on the sea are my prey.",
        'Might. In life as in war, the stronger force wins.',
        'Retribution. The rich need to be shown what life and death are like in the gutters.',
    ],
    'good': [
        'Charity. I always try to help those in need, no matter what the personal cost.',
        'Charity. I distribute money I acquire to the people who really need it.',
        'Friendship. Material goods come and go. Bonds of friendship last forever.',
        'Charity. I steal from the wealthy so that I can help people in need.',
        "Redemption. There's a spark of good in everyone.",
        'Beauty. When I perform, I make the world better than it was.',
        'Respect. People deserve to be treated with dignity and respect.',
        'Generosity. My talents were given to me so that I could use them to benefit the world.',
        'Greater Good. My gifts are meant to be shared with all, not used for my own benefit.',
        'Respect. Respect is due to me because of my position, but all people regardless of station deserve to be treated with dignity.',
        'Noble Obligation. It is my duty to protect and care for the people beneath me.',
        "Greater Good. It is each person's responsibility to make the most happiness for the whole tribe.",
        'Beauty. What is beautiful points us beyond itself toward what is true.',
        'Respect. The thing that keeps a ship together is mutual respect between captain and crew.',
        'Greater Good. Our lot is to lay down our lives in defense of others.',
        'Respect. All people, rich or poor, deserve respect.',
    ],
    'lawful': [
        'Faith. I trust that my deity will guide my actions. I have faith that if I work hard, things will go well.',
        'Tradition. The ancient traditions of worship and sacrifice must be preserved and upheld.',
        "Power. I hope to one day rise to the top of my faith's religious hierarchy.",
        "Fairness. I never target people who can't afford to lose a few coins.",
        "Honor. I don't steal from others in the trade.",
        'Tradition. The stories, legends, and songs of the past must never be forgotten.',
        'Fairness. No one should get preferential treatment before the law, and no one is above the law.',
        'Community. It is the duty of all civilized people to strengthen the bonds of community and the security of civilization.',
        'Logic. Emotions must not cloud our sense of what is right and true, or our logical thinking.',
        'Responsibility. It is my duty to respect the authority of those above me, just as those below me must respect mine.',
        'Honor. If I dishonor myself, I dishonor my whole clan.',
        'Logic. Emotions must not cloud our logical thinking.',
        'Fairness. We all do the work, so we all share in the rewards.',
        'Responsibility. I do what I must and obey just authority.',
        'Community. We have to take care of each other, because no one else is going to do it.',
    ],
    'neutral': [
        "People. I'm loyal to my friends, not to any ideals, and everyone else can take a trip down the Styx for all I care.",
        "People. I like seeing the smiles on people's faces when I perform. That's all that matters.",
        "Sincerity. There's no good pretending to be something I'm not.",
        "People. I'm committed to the people I care about, not to ideals.",
        'Live and Let Live. Meddling in the affairs of others only causes trouble.',
        'Nature. The natural world is more important than all the constructs of civilization.',
        'Knowledge. The path to power and self-improvement is through knowledge.',
        "People. I'm committed to my crewmates, not to ideals.",
        "Ideals aren't worth killing for or going to war for.",
        "People. I help people who help me--that's what keeps us alive.",
    ]
}

TRAITS = [
    "I idolize a particular hero of my faith and constantly refer to that person's deeds and example.",
    'I can find common ground between the fiercest enemies, empathizing with them and always working toward peace.',
    'I see omens in every event and action. The gods try to speak to us, we just need to listen.',
    'Nothing can shake my optimistic attitude.',
    'I quote (or misquote) the sacred texts and proverbs in almost every situation.',
    'I am tolerant (or intolerant) of other faiths and respect (or condemn) the worship of other gods.',
    "I've enjoyed fine food, drink, and high society among my temple's elite. Rough living grates on me.",
    "I've spent so long in the temple that I have little practical experience dealing with people in the outside world.",
    'I fall in and out of love easily, and am always pursuing someone.',
    'I have a joke for every occasion, especially occasions where humor is inappropriate.',
    'Flattery is my preferred trick for getting what I want.',
    "I'm a born gambler who can't resist taking a risk for a potential payoff.",
    "I lie about almost everything, even when there's no good reason to.",
    'Sarcasm and insults are my weapons of choice.',
    'I keep multiple holy symbols on me and invoke whatever deity might come in useful at any given moment.',
    'I pocket anything I see that might have some value.',
    'I always have plan for what to do when things go wrong.',
    'I am always calm, no matter what the situation. I never raise my voice or let my emotions control me.',
    'The first thing I do in a new place is note the locations of everything valuable--or where such things could be hidden.',
    'I would rather make a new friend than a new enemy.',
    'I am incredibly slow to trust. Those who seem the fairest often have the most to hide.',
    "I don't pay attention to the risks in a situation. Never tell me the odds.",
    "The best way to get me to do something is to tell me I can't do it.",
    'I blow up at the slightest insult.',
    'I know a story relevant to almost every situation.',
    'Whenever I come to a new place, I collect local rumors and spread gossip.',
    "I'm a hopeless romantic, always searching for that 'special someone'.",
    'Nobody stays angry at me or around me for long, since I can defuse any amount of tension.',
    'I love a good insult, even one directed at me.',
    "I get bitter if I'm not the center of attention.",
    "I'll settle for nothing less than perfection.",
    'I change my mood or my mind as quickly as I change key in a song.',
    'I judge people by their actions, not their words.',
    "If someone is in trouble, I'm always willing to lend help.",
    'When I set my mind to something, I follow through no matter what gets in my way.',
    'I have a strong sense of fair play and always try to find the most equitable solution to arguments.',
    "I'm confident in my own abilities and do what I can to instill confidence in others.",
    'Thinking is for other people. I prefer action.',
    'I misuse long words in an attempt to sound smarter.',
    'I get bored easily. When am I going to get on with my destiny.',
    "I believe that everything worth doing is worth doing right. I can't help it--I'm a perfectionist.",
    "I'm a snob who looks down on those who can't appreciate fine art.",
    'I always want to know how things work and what makes people tick.',
    "I'm full of witty aphorisms and have a proverb for every occasion.",
    "I'm rude to people who lack my commitment to hard work and fair play.",
    'I like to talk at length about my profession.',
    "I don't part with my money easily and will haggle tirelessly to get the best deal possible.",
    "I'm well known for my work, and I want to make sure everyone appreciates it. I'm always taken aback when people haven't heard of me.",
    "I've been isolated for so long that I rarely speak, preferring gestures and the occasional grunt.",
    'I am utterly serene, even in the face of disaster.',
    'The leader of my community has something wise to say on every topic, and I am eager to share that wisdom.',
    'I feel tremendous empathy for all who suffer.',
    "I'm oblivious to etiquette and social expectations.",
    'I connect everything that happens to me to a grand cosmic plan.',
    'I often get lost in my own thoughts and contemplations, becoming oblivious to my surroundings.',
    'I am working on a grand philosophical theory and love sharing my ideas.',
    'My eloquent flattery makes everyone I talk to feel like the most wonderful and important person in the world.',
    'The common folk love me for my kindness and generosity.',
    'No one could doubt by looking at my regal bearing that I am a cut above the unwashed masses.',
    'I take great pains to always look my best and follow the latest fashions.',
    "I don't like to get my hands dirty, and I won't be caught dead in unsuitable accommodations.",
    'Despite my birth, I do not place myself above other folk. We all have the same blood.',
    'My favor, once lost, is lost forever.',
    'If you do me an injury, I will crush you, ruin your name, and salt your fields.',
    "I'm driven by a wanderlust that led me away from home.",
    'I watch over my friends as if they were a litter of newborn pups.',
    "I once ran twenty-five miles without stopping to warn my clan of an approaching orc horde. I'd do it again if I had to.",
    'I have a lesson for every situation, drawn from observing nature.',
    "I place no stock in wealthy or well-mannered folk. Money and manners won't save you from a hungry owlbear.",
    "I'm always picking things up, absently fiddling with them, and sometimes accidentally breaking them.",
    'I feel far more comfortable around animals than people.',
    'I was, in fact, raised by wolves.',
    'I use polysyllabic words to convey the impression of great erudition.',
    "I've read every book in the world's greatest libraries--or like to boast that I have.",
    "I'm used to helping out those who aren't as smart as I am, and I patiently explain anything and everything to others.",
    "There's nothing I like more than a good mystery.",
    "I'm willing to listen to every side of an argument before I make my own judgment.",
    'I...speak...slowly...when talking...to idiots...which...almost...everyone...is...compared ...to me.',
    'I am horribly, horribly awkward in social situations.',
    "I'm convinced that people are always trying to steal my secrets.",
    'My friends know they can rely on me, no matter what.',
    'I work hard so that I can play hard when the work is done.',
    'I enjoy sailing into new ports and making new friends over a flagon of ale.',
    'I stretch the truth for the sake of a good story.',
    'To me, a tavern brawl is a nice way to get to know a new city.',
    'I never pass up a friendly wager.',
    'My language is as foul as an otyugh nest.',
    'I like a job well done, especially if I can convince someone else to do it.',
    "I'm always polite and respectful.",
    "I'm haunted by memories of war. I can't get the images of violence out of my mind.",
    "I've lost too many friends, and I'm slow to make new ones.",
    "I'm full of inspiring and cautionary tales from my military experience relevant to almost every combat situation.",
    'I can stare down a hellhound without flinching.',
    'I enjoy being strong and like breaking things.',
    'I have a crude sense of humor.',
    'I face problems head-on. A simple direct solution is the best path to success.',
    'I hide scraps of food and trinkets away in my pockets.',
    'I ask a lot of questions.',
    'I like to squeeze into small places where no one else can get to me.',
    'I sleep with my back to a wall or tree, with everything I own wrapped in a bundle in my arms.',
    'I eat like a pig and have bad manners.',
    "I think anyone who's nice to me is hiding evil intent.",
    "I don't like to bathe.",
    'I bluntly say what other people are hinting or hiding.',
]

FLAWS = [
    'I judge others harshly, and myself even more severely.',
    "I put too much trust in those who wield power within my temple's hierarchy.",
    'My piety sometimes leads me to blindly trust those that profess faith in my god.',
    'I am inflexible in my thinking.',
    'I am suspicious of strangers and suspect the worst of them.',
    'Once I pick a goal, I become obsessed with it to the detriment of everything else in my life.',
    "I can't resist a pretty face.",
    "I'm always in debt. I spend my ill-gotten gains on decadent luxuries faster than I bring them in.",
    "I'm convinced that no one could ever fool me in the way I fool others.",
    "I'm too greedy for my own good. I can't resist taking a risk if there's money involved.",
    "I can't resist swindling people who are more powerful than me.",
    "I hate to admit it and will hate myself for it, but I'll run and preserve my own hide if the going gets tough.",
    "When I see something valuable, I can't think about anything but how to steal it.",
    'When faced with a choice between money and my friends, I usually choose the money.',
    "If there's a plan, I'll forget it. If I don't forget it, I'll ignore it.",
    "I have a 'tell' that reveals when I'm lying.",
    'I turn tail and run when things go bad.',
    "An innocent person is in prison for a crime that I committed. I'm okay with that.",
    "I'll do anything to win fame and renown.",
    "I'm a sucker for a pretty face.",
    'A scandal prevents me from ever going home again. That kind of trouble seems to follow me around.',
    'I once satirized a noble who still wants my head. It was a mistake that I will likely repeat.',
    'I have trouble keeping my true feelings hidden. My sharp tongue lands me in trouble.',
    'Despite my best efforts, I am unreliable to my friends.',
    'The tyrant who rules my land will stop at nothing to see me killed.',
    "I'm convinced of the significance of my destiny, and blind to my shortcomings and the risk of failure.",
    'The people who knew me when I was young know my shameful secret, so I can never go home again.',
    'I have a weakness for the vices of the city, especially hard drink.',
    'Secretly, I believe that things would be better if I were a tyrant lording over the land.',
    'I have trouble trusting in my allies.',
    "I'll do anything to get my hands on something rare or priceless.",
    "I'm quick to assume that someone is trying to cheat me.",
    'No one must ever learn that I once stole money from guild coffers.',
    "I'm never satisfied with what I have--I always want more.",
    'I would kill to acquire a noble title.',
    "I'm horribly jealous of anyone who outshines my handiwork. Everywhere I go, I'm surrounded by rivals.",
    "Now that I've returned to the world, I enjoy its delights a little too much.",
    'I harbor dark bloodthirsty thoughts that my isolation failed to quell.',
    'I am dogmatic in my thoughts and philosophy.',
    'I let my need to win arguments overshadow friendships and harmony.',
    "I'd risk too much to uncover a lost bit of knowledge.",
    "I like keeping secrets and won't share them with anyone.",
    'I secretly believe that everyone is beneath me.',
    'I hide a truly scandalous secret that could ruin my family forever.',
    "I too often hear veiled insults and threats in every word addressed to me, and I'm quick to anger.",
    'I have an insatiable desire for carnal pleasures.',
    'In fact, the world does revolve around me.',
    'By my words and actions, I often bring shame to my family.',
    'I am too enamored of ale, wine, and other intoxicants.',
    "There's no room for caution in a life lived to the fullest.",
    "I remember every insult I've received and nurse a silent resentment toward anyone who's ever wronged me.",
    'I am slow to trust members of other races',
    'Violence is my answer to almost any challenge.',
    "Don't expect me to save those who can't save themselves. It is nature's way that the strong thrive and the weak perish.",
    'I am easily distracted by the promise of information.',
    'Most people scream and run when they see a demon. I stop and take notes on its anatomy.',
    'Unlocking an ancient mystery is worth the price of a civilization.',
    'I overlook obvious solutions in favor of complicated ones.',
    'I speak without really thinking through my words, invariably insulting others.',
    "I can't keep a secret to save my life, or anyone else's.",
    "I follow orders, even if I think they're wrong.",
    "I'll say anything to avoid having to do extra work.",
    'Once someone questions my courage, I never back down no matter how dangerous the situation.',
    "Once I start drinking, it's hard for me to stop.",
    "I can't help but pocket loose coins and other trinkets I come across.",
    'My pride will probably lead to my destruction',
    'The monstrous enemy we faced in battle still leaves me quivering with fear.',
    'I have little respect for anyone who is not a proven warrior.',
    'I made a terrible mistake in battle that cost many lives--and I would do anything to keep that mistake secret.',
    'My hatred of my enemies is blind and unreasoning.',
    'I obey the law, even if the law causes misery.',
    "I'd rather eat my armor than admit when I'm wrong.",
    "If I'm outnumbered, I always run away from a fight.",
    "Gold seems like a lot of money to me, and I'll do just about anything for more of it.",
    'I will never fully trust anyone other than myself.',
    "I'd rather kill someone in their sleep than fight fair.",
    "It's not stealing if I need it more than someone else.",
    "People who don't take care of themselves get what they deserve.",
]



'''
towniesim.util
==============

Utility functions
'''
import re
from random import uniform, randint, seed, gauss

RE_PERCENT = re.compile(r'(\d+(?:\.\d*)?)%?\s*(.*)')


def initialize_seed(n):
    '''
    Initialize seed for deterministic output.
    '''
    if n is None:
        seed()
    else:
        seed(n)


def nds(number_dice, sides):
    '''
    short for number_dice_sides
    Another convenience dice roller like ``nds(3, 6)`` for 3d6.
    '''
    summed = 0
    for i in range(number_dice):
        summed += randint(1, sides)
    return summed


def roll(s):
    '''
    Roll dice like ``roll('3d6')``
    '''
    s = s.strip().lower()
    num, sides = s.split('d')
    return nds(int(num), int(sides))


def percents_from_list(lst):
    '''
    Utility function to derive percents and values for ``random_from_list``
    '''
    probs = {}
    for item in lst:
        match = RE_PERCENT.match(item)
        if match is None and len(lst) == 1:
            return item
        elif match is None:
            raise ValueError('In list of percents, {} doesnt match pattern '
                             'like "50% elf"'.format(item))
        percent = float(match.group(1))
        value = match.group(2)
        probs[value] = percent
    return probs


def random_from_list(lst):
    '''
    This will pick a random item from a list that matches string patterns like:
    ['50% human', '45% elf', '5% dwarf']
    It can also work with arbitrary numbers like:
    ['5 human', '1 elf', '1 dwarf']
    Or:
    [('human', 5), ('elf', 1)]
    or:
    {'human': 5, 'elf': 1}

    This is used so configs can be created and parsed with configobj like:
    tavern_races=50% human, 30% elf, 20% dwarf
    '''
    if isinstance(lst, dict):
        lst = list(lst.items())
    if lst and isinstance(lst[0], (list, tuple)):
        probs = sorted(lst, key=lambda x: x[1], reverse=True)
    else:
        probs = sorted(percents_from_list(lst).items(), key=lambda x: x[1],
                       reverse=True)
    inc = []
    summ = 0
    # build this list so we can incrementally check if rand < summ
    for value, num in probs:
        summ += num
        inc += [(value, summ)]

    total = sum(x[1] for x in probs)
    rand = uniform(0, total)
    for value, num in inc:
        if rand <= num:
            return value
    return value


def le_switch(d, n):
    for i, v in sorted(d.items()):
        if n <= i:
            return v
    return None


def even_switch(d, n):
    summ = 0
    inc = []
    if isinstance(d, dict):
        lst = d.items()
    else:
        lst = d
    for num, value in lst:
        summ += num
        inc += [(value, summ)]
    for value, num in inc:
        if n <= num:
            return value
    return None


def gauss_mm(mu, sigma, mn=None, mx=None):
    g = gauss(mu, sigma)
    if mx is not None:
        g = min(g, mx)
    if mn is not None:
        g = max(g, mn)
    return g


def friendly_wealth(wealth):
    return {
        0: 'unknown',
        1: 'lower class',
        2: 'middle class',
        3: 'upper class',
    }.get(wealth, 'unknown[{}]'.format(wealth))


def friendly_racism(racism_num):
    ''' lol '''
    return le_switch({
        -40: 'loves',
        -25: 'likes',
        -15: 'friendly to',
        15: None,
        25: 'unfriendly to',
        40: 'dislikes',
        50: 'hates',
    }, racism_num)


def friendly_age(age):
    years = int(age)
    months = round((age - int(age)) * 12)
    return '{} years {} months'.format(years, months)



import os
from setuptools import setup

# clouddistrict
# Generate occupants and occupations for your custom village, town or city! (for D&D purposes or related) (please do not imbibe clouddistrict while under the influence of other medications)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="clouddistrict",
    version="0.0.4",
    description="Generate occupants and occupations for your custom village, town or city! (for D&D purposes or related) (please do not imbibe clouddistrict while under the influence of other medications)",
    author="Johan Nestaas",
    author_email="johannestaas@gmail.com",
    license="GPLv3+",
    keywords="",
    url="https://www.bitbucket.org/johannestaas/clouddistrict",
    packages=['clouddistrict'],
    package_dir={'clouddistrict': 'clouddistrict'},
    long_description=read('README.rst'),
    classifiers=[
        # 'Development Status :: 1 - Planning',
        # 'Development Status :: 2 - Pre-Alpha',
        'Development Status :: 3 - Alpha',
        # 'Development Status :: 4 - Beta',
        # 'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'License :: OSI Approved :: GNU General Public License v3 or later '
        '(GPLv3+)',
        'Environment :: Console',
        'Environment :: X11 Applications :: Qt',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
    ],
    install_requires=[
        'pyyaml',
    ],
    entry_points={
        'console_scripts': [
            'clouddistrict=clouddistrict:main',
        ],
    },
    zip_safe=False,
    package_data={
        'clouddistrict': ['names/*.txt'],
    },
    include_package_data=True,
)



'''
towniesim.family
================
'''
from enum import Enum
from random import choice, uniform
from itertools import chain
from .race import Age
from .npc import NPC
from .util import even_switch, nds, gauss_mm


class FamilyType(Enum):
    normal = 'husband, wife and children'
    single_father = 'father and children'
    single_mother = 'mother and children'
    siblings = 'siblings'
    single_male = 'single male'
    single_female = 'single female'


def random_gold(wealth):
    if wealth == 0:
        return 0
    if wealth == 1:
        return int(uniform(500, 1500))
    if wealth == 2:
        return int(uniform(2500, 5000))
    if wealth == 3:
        return int(uniform(5000, 50000))
    return uniform(2500, 5000)


class Family:
    COUNT = 0
    OBJS = {}

    @classmethod
    def next(cls):
        cls.COUNT += 1
        return cls.COUNT

    TYPES = (
        (50, FamilyType.normal),
        (15, FamilyType.single_male),
        (15, FamilyType.single_female),
        (10, FamilyType.siblings),
        (5, FamilyType.single_father),
        (5, FamilyType.single_mother),
    )

    def __repr__(self):
        fam = [repr(m) for m in self.members()]
        fam = '\n - ' + '\n - '.join(fam)
        return 'Family({}\n)'.format(fam)

    def __init__(self, race=None, wealth=0):
        self.id = Family.next()
        self.idstr = 'family#{}'.format(self.id).lower()
        Family.OBJS[self.idstr] = self
        self.name = None
        self.mother = None
        self.father = None
        self.children = []
        self.family_type = even_switch(self.TYPES, nds(1, 100))
        self.race = race
        self.wealth = wealth
        self.gold = random_gold(wealth)
        if self.family_type == FamilyType.normal:
            self.father = self.create_male(age=Age.random_parent())
            self.mother = self.create_female(age=Age.random_parent())
            self.mother.take_last_name(self.father)
            self.children = self.create_children()
            for child in self.children:
                child.take_last_name(self.father)
            self.name = self.father.get_last_name()
        elif self.family_type == FamilyType.single_father:
            self.father = self.create_male(age=Age.random_parent())
            self.children = self.create_children()
            for child in self.children:
                child.take_last_name(self.father)
            self.name = self.father.get_last_name()
        elif self.family_type == FamilyType.single_mother:
            self.mother = self.create_female(age=Age.random_parent())
            self.children = self.create_children()
            for child in self.children:
                child.take_last_name(self.mother)
            self.name = self.mother.get_last_name()
        elif self.family_type == FamilyType.single_male:
            self.father = self.create_male(age=Age.random_notchild())
            self.name = self.father.get_last_name()
        elif self.family_type == FamilyType.single_female:
            self.mother = self.create_female(age=Age.random_notchild())
            self.name = self.mother.get_last_name()
        elif self.family_type == FamilyType.siblings:
            self.children = self.create_children(min_num=2)
            children = list(self.children)
            for child in children[1:]:
                child.take_last_name(children[0])
            self.name = children[0].get_last_name()
        else:
            raise ValueError('No family type: {!r}'.format(self.family_type))
        self.link_family()

    @property
    def father(self):
        if self._father is None:
            return None
        return NPC.OBJS[self._father]

    @father.setter
    def father(self, val):
        if isinstance(val, NPC):
            self._father = val.idstr
        else:
            self._father = val

    @property
    def mother(self):
        if self._mother is None:
            return None
        return NPC.OBJS[self._mother]

    @mother.setter
    def mother(self, val):
        if isinstance(val, NPC):
            self._mother = val.idstr
        else:
            self._mother = val

    @property
    def children(self):
        return [NPC.OBJS[child] for child in self._children]

    @children.setter
    def children(self, val):
        self._children = []
        for child in val:
            if isinstance(child, NPC):
                self._children.append(child.idstr)
            else:
                self._children.append(child)

    def create_male(self, age=None, wealth=None):
        age = age or Age.random()
        new = NPC(race=self.race, gender='male', age=age, wealth=wealth)
        return new

    def create_female(self, age=None, wealth=None):
        age = age or Age.random()
        new = NPC(race=self.race, gender='female', age=age, wealth=wealth)
        return new

    def create_children(self, min_num=1):
        children = set()
        for i in range(int(gauss_mm(2, 1, min_num, 9))):
            age = Age.random_age(parents=[self.mother, self.father])
            func = choice([self.create_male, self.create_female])
            new = func(age=age, wealth=self.wealth)
            children.add(new)
        return children

    def link_family(self):
        if self.father:
            self.father.lover = self.mother
            self.father.family = self
        if self.mother:
            self.mother.lover = self.father
            self.mother.family = self
        for child in self.children:
            child.family = self

    def members(self):
        if self.father:
            yield self.father
        if self.mother:
            yield self.mother
        if self.children:
            for child in self.children:
                yield child

    def assign_workplace(self, workplace):
        members = set()
        jobtype = choice(workplace.JOBS)
        for mem in self.members():
            if mem.get_age() in (Age.child, Age.venerable):
                continue
            members.add(mem)
            job = jobtype(location=workplace)
            mem.set_job(job)

    @property
    def owned_buildings(self):
        return list(chain(*[x.owned_buildings for x in self.members()]))

    @property
    def owned_residences(self):
        return list(chain(*[x.owned_residences for x in self.members()]))

    def oldest(self):
        return max(list(self.members()), key=lambda x: x.age)

    def oldest_nonowner(self):
        members = [
            x for x in self.members()
            if not (x.owned_buildings or x.get_age() == Age.child)
        ]
        if not members:
            return None
        return max(members, key=lambda x: x.age)

    def working(self):
        return [
            x for x in self.members()
            if x.get_age() not in (Age.child, Age.venerable)
        ]

    def can_afford(self, building):
        return self.gold >= building.COST

    def buy(self, building):
        from .building import Building
        from .residence import Residence
        self.gold -= building.COST
        owner = self.oldest_nonowner()
        if owner is None:
            if not self.working():
                owner = self.oldest()
            else:
                owner = choice(self.working())
        building.owner = owner
        if isinstance(building, Building):
            owner.owned_buildings += [building]
        elif isinstance(building, Residence):
            owner.owned_residences += [building]
        else:
            raise ValueError('cant handle building type {!r}'.format(building))
        return owner



'''
towniesim.residence
===================
'''
from random import choice


class MetaResidence(type):

    RESIDENCES = {}

    def __new__(cls, name, bases, dct):
        new_cls = super(MetaResidence, cls).__new__(cls, name, bases, dct)
        if name == 'Residence':
            return new_cls
        MetaResidence.RESIDENCES[dct['NAME']] = new_cls
        return new_cls


class Residence(metaclass=MetaResidence):
    COUNT = 0
    OBJS = {}

    @classmethod
    def next(cls):
        cls.COUNT += 1
        return cls.COUNT

    def __init__(self, owner=None, wealth=0, **kwargs):
        self.id = Residence.next()
        self.idstr = '{}#{}'.format(self.NAME, self.id).lower()
        Residence.OBJS[self.idstr] = self
        self.wealth = wealth
        self.owner = owner

    @property
    def owner(self):
        from .npc import NPC
        if self._owner is None:
            return None
        return NPC.OBJS[self._owner]

    @owner.setter
    def owner(self, val):
        from .npc import NPC
        if isinstance(val, NPC):
            self._owner = val.idstr
        else:
            self._owner = val

    @classmethod
    def get_residences(cls, max_level=None):
        rezs = MetaResidence.RESIDENCES.values()
        if max_level is not None:
            rezs = [r for r in rezs if r.LEVEL <= max_level]
        return rezs

    @classmethod
    def get_residence(cls, name):
        return MetaResidence.RESIDENCES[name]

    @classmethod
    def rand(cls, wealth=None):
        if not wealth:
            return choice([Cottage, Home, TownHouse])
        if wealth == 1:
            return choice([Cottage, Home])
        if wealth == 2:
            return choice([Cottage, Home, TownHouse])
        if wealth == 3:
            return choice([Cottage] + [Home, TownHouse] * 9 + [Mansion])
        return Home

    def dump(self):
        residents = ''
        for res in self.owner.family.members():
            residents += '- {res!r} {resid}\n'.format(res=res, resid=res.idstr)
        return '''{lastname}'s {name}
ID: {id}
Owner: {owner!r}
Owner ID: {ownerid}
Residents:\n{residents}
'''.format(
            lastname=self.owner.name.split()[-1],
            name=self.NAME,
            id=self.idstr,
            owner=self.owner,
            ownerid=self.owner.idstr,
            residents=residents,
    )


class Cottage(Residence):
    NAME = 'cottage'
    LEVEL = 1


class Home(Residence):
    NAME = 'home'
    LEVEL = 1


class TownHouse(Residence):
    NAME = 'townhouse'
    LEVEL = 2


class Mansion(Residence):
    NAME = 'mansion'
    LEVEL = 3
